#include "iri_joints_client_alg_node.h"

IriJointsClientAlgNode::IriJointsClientAlgNode(void) :
  algorithm_base::IriBaseAlgorithm<IriJointsClientAlgorithm>(),
  joints_module("joints_module",ros::this_node::getName())
{
  //init class attributes if necessary
  //this->loop_rate_ = 2;//in [Hz]

  // [init publishers]
  
  // [init subscribers]
  
  // [init services]
  
  // [init clients]
  std::vector<std::string>().swap(this->joint_names);
  std::vector<double>().swap(this->angles);
  std::vector<double>().swap(this->vel);
}

IriJointsClientAlgNode::~IriJointsClientAlgNode(void)
{
  // [free dynamic memory]
}

void IriJointsClientAlgNode::mainNodeThread(void)
{
  // [fill msg structures]
  
  // [fill srv structure and make request to the server]

  // [publish messages]

}

/*  [subscriber callbacks] */

/*  [service callbacks] */

/*  [nav callbacks] */

void IriJointsClientAlgNode::node_config_update(Config &config, uint32_t level)
{
  this->alg_.lock();
  if (config.send_point_to)
  {
    config.send_point_to = false;
    if (this->joints_module.point_to(config.point_x, config.point_y, config.point_z, config.vel_max))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Point_to called");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while calling point_to");
  }
  if (config.send_ik_tol)
  {
    config.send_ik_tol = false;
    //if (this->joints_module.set_tolerance(config.ik_tol))
    if (this->joints_module.set_parameter("max_ik_tol", config.ik_tol))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Tolerance set");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while setting tolerance");
  }
  if(config.cancel_point_to)
  {
    config.cancel_point_to = false;
    ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Cancel action point_to");
    this->joints_module.cancel_point_to();
  }
  if (config.send_base_frame)
  {
    config.send_base_frame = false;
    if (this->joints_module.set_base_frame(config.base_frame))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Base frame set");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while setting base frame");
  }
  if (config.send_urdf_param)
  {
    config.send_urdf_param = false;
    if (this->joints_module.set_urdf_param(config.urdf_param))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Urdf param set");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while setting urdf param");
  }

  if (config.start_tracking)
  {
    config.start_tracking = false;
    this->joints_module.enable_tracker();
  }

  if (config.stop_tracking)
  {
    config.stop_tracking = false;
    this->joints_module.cancel_tracker();
  }

  if (config.update_target)
  {
    config.update_target = false;
    if (this->joints_module.target_update(config.point_x, config.point_y, config.point_z))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Update target");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while updating target");
  }

  if (config.add_joint)
  {
    ROS_INFO_STREAM("IriJointsClientAlgNode::node_config_update -> Added joint");
    config.add_joint = false;
    this->joint_names.push_back(config.joint_name);
    this->angles.push_back(config.joint_angle);
    this->vel.push_back(config.joint_vel);
  }
  if (config.send_move_to_angles)
  {
    config.send_move_to_angles = false;
    if (this->joints_module.move_to_angles(this->joint_names, this->angles, this->vel))
      ROS_DEBUG("IriJointsClientAlgNode::node_config_update -> Move_to_angles called");
    else
      ROS_ERROR("IriJointsClientAlgNode::node_config_update -> Error while calling move_to_angles");
    std::vector<std::string>().swap(this->joint_names);
    std::vector<double>().swap(this->angles);
    std::vector<double>().swap(this->vel);
  }
  if (config.cancel_move_to_angles)
  {
    config.cancel_move_to_angles = false;
    this->joints_module.cancel_move_to_angles();
  }
  this->alg_.unlock();
}

void IriJointsClientAlgNode::addNodeDiagnostics(void)
{
}

/* main function */
int main(int argc,char *argv[])
{
  return algorithm_base::main<IriJointsClientAlgNode>(argc, argv, "iri_joints_client_alg_node");
}
